package org.example;

import javafx.util.Pair;

import java.util.*;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.*;

public class GroupByExamples {

  public static void main(String[] args) {

    List<Person> list = Arrays.asList(new Person(1, "Alex", 100d, new Department(1, "HR")),
            new Person(2, "Brian", 200d, new Department(1, "HR")),
            new Person(3, "Charles", 900d, new Department(2, "Finance")),
            new Person(4, "David", 200d, new Department(2, "Finance")),
            new Person(5, "Edward", 200d, new Department(2, "Finance")),
            new Person(6, "Frank", 800d, new Department(3, "ADMIN")),
            new Person(7, "George", 900d, new Department(3, "ADMIN")));

    System.out.println("GroupByExamples by Simple Condition :" + simpleCondition(list));

    System.out.println(" collect only the person ids in all departments" + personIds(list));

    System.out.println("GroupByExamples by Complex Condition" + complex(list));

    System.out.println("GroupByExamples with Counting" + count(list));

    System.out.println(" GroupByExamples with Average" + average(list));

    System.out.println("   GroupByExamples with Max/Min" + max(list));

    System.out.println("GroupByExamples with Filtering" + filter(list));


    Map<Integer, Department> departmentMap = list
        .stream()
        .collect(Collectors.toMap(Person::getId, Person::getDepartment));
  }

  private static Map<Department, List<Person>> filter(final List<Person> list) {
    return list
        .stream()
        .filter(person -> person.getSalary() > 100)
        .collect(Collectors.groupingBy(Person::getDepartment));
  }

  private static Map<Department, Optional<Person>> max(final List<Person> list) {
    return list
        .stream()
        .collect(Collectors.groupingBy(Person::getDepartment, maxBy(Comparator.comparing(Person::getSalary))));
  }

  private static Map<Department, Double> average(final List<Person> list) {
    return list.stream().collect(Collectors.groupingBy(Person::getDepartment, averagingDouble(Person::getSalary)));
  }

  private static Map<Department, Long> count(final List<Person> list) {
    return list.stream().collect(Collectors.groupingBy(Person::getDepartment, counting()));
  }

  private static Map<Object, List<Integer>> complex(final List<Person> list) {
    return list
        .stream()
        .collect(Collectors.groupingBy(person -> new Pair<>(person.getSalary(), person.getSalary()),
            Collectors.mapping(Person::getId, toList())));

  }

  private static Map<Department, List<Integer>> personIds(final List<Person> list) {
    return list.stream().collect(Collectors.groupingBy(Person::getDepartment, mapping(Person::getId, toList())));
  }

  private static Map<Department, List<Person>> simpleCondition(final List<Person> list) {
    return list.stream().collect(Collectors.groupingBy(Person::getDepartment));
  }


}
